import os
import unittest
import flask_migrate
import flask_testing

from ui import config
from ui.app import app
from ui.models import Build, Runner, db

class UsesApp(flask_testing.TestCase):
    def create_app(self):
        return app

class UsesModels(UsesApp):
    def setUp(self):
        flask_migrate.upgrade(revision="head")

    def tearDown(self):
        flask_migrate.downgrade(revision="base")

class TestModels(UsesModels):

    def test_build(self):
        build = Build(build_id=1)
        db.session.add(build)
        db.session.commit()

        assert build in db.session

    def test_get_or_create_build_by_id(self):
        build = Build(build_id=1)
        db.session.add(build)
        db.session.commit()

        assert Build.get_or_create_by_id(1).build_id == 1
        assert Build.get_or_create_by_id(2).build_id == 2

    def test_runner(self):
        runner = Runner(runner_id="foobar")
        db.session.add(runner)
        db.session.commit()

        assert runner in db.session

    def test_get_or_create_runner_by_id(self):
        runner = Runner(runner_id="foobar", runner_name="foobar")
        db.session.add(runner)
        db.session.commit()

        assert Runner.get_or_create_by_id('foobar').runner_name == "foobar"
        assert Runner.get_or_create_by_id("foobaz").runner_name == None

    def test_runner_build_rel(self):
        runner = Runner(runner_id="foobar", runner_name="foobar")
        build = Build(build_id=1, build_runner=runner)
        db.session.add(runner)
        db.session.add(build)
        db.session.commit()

        assert build.build_runner == runner
        assert build in runner.builds

class TestWebhooks(UsesModels):

    def test_invalid_token(self):
        with open("tests/pipeline_webhook.json", "r") as f:
            response = self.client.post("/webhook", headers={'X-Gitlab-Event': 'Pipeline Hook', 'X-Gitlab-Token': 'invalidsecret'}, data=f.read(), content_type='application/json')
        assert response.status_code == 403

    def test_pipeline_webhook(self):
        with open("tests/pipeline_webhook.json", "r") as f:
            response = self.client.post("/webhook", headers={'X-Gitlab-Event': 'Pipeline Hook', 'X-Gitlab-Token': 'secret'}, data=f.read(), content_type='application/json')
        assert response.status_code == 200
        build = Build.query.filter_by(build_id=1).first()
        assert build.build_id == 1
        assert build.build_runner_id == "foobar"

if __name__ == "__main__":
    unittest.main()
