from flask import abort

from ui import config
from ui.gitlab import api
from ui.models import Build, Runner, db

def process(request):
    if request.headers.get('X-Gitlab-Token', None) != config.GITLAB_WEBHOOK_TOKEN:
        abort(403)
    webhook_type = request.headers.get('X-Gitlab-Event')
    data = request.get_json()
    if webhook_type == 'Pipeline Hook':
        pipeline = data.get('object_attributes')
        stages = data.get('builds')
        build = Build.get_or_create_by_id(pipeline.get('id'))
        build.status = pipeline.get('status')
        build.duration = pipeline.get('duration')
        for variable in pipeline.get("variables"):
            if variable.get('key') == "VERSION":
                build.build_version = variable.get("value")
            elif variable.get("key") == "DEVICE":
                build.build_device = variable.get("value")
            elif variable.get("key") == "TYPE":
                build.build_type = variable.get("value")
        build_stage = {}
        for stage in stages:
            if stage.get('name') == 'build':
                build_stage = stage
        if build_stage.get("runner"):
            build.build_runner = Runner.get_or_create_by_id(build_stage['runner'])
        db.session.add(build)
        db.session.commit()
